import itertools
from dataclasses import asdict

from gwdc_python import GWDC
from gwdc_python.files import FileReference, FileReferenceList
from gwdc_python.helpers import TimeRange
from gwdc_python.utils import rename_dict_keys
from gwdc_python.logger import create_logger

from .viterbi_job import ViterbiJob
from .utils.file_download import _download_files, _save_file_map_fn, _get_file_map_fn
from .settings import GWLAB_VITERBI_ENDPOINT

logger = create_logger(__name__)


class GWLabViterbi:
    def __init__(self, token, endpoint=GWLAB_VITERBI_ENDPOINT):
        self.client = GWDC(token=token, endpoint=endpoint)
        self.request = self.client.request

    def start_viterbi_job(self, job_name, job_description, private, data_input, data_params, search_params):
        """Start a viterbi job in the most basic possible way

        Parameters
        ----------
        variables : dict
            Dictionary containing the keys and values for each of the required input fields to start a job
        """
        query = """
            mutation NewViterbiJob($input: ViterbiJobMutationInput!){
                newViterbiJob (input: $input) {
                    result {
                        jobId
                    }
                }
            }
        """

        variables = {
            "input": {
                "start": {
                    "name": job_name,
                    "description": job_description,
                    "private": private,
                },
                "data": asdict(data_input),
                "data_parameters": asdict(data_params),
                "search_parameters": asdict(search_params),
            }
        }

        data = self.request(query=query, variables=variables)

        job_id = data['new_viterbi_job']['result']['job_id']
        return self.get_job_by_id(job_id)

    def _get_job_model_from_query(self, query_data):
        if not query_data:
            return None

        return ViterbiJob(
            client=self,
            **rename_dict_keys(
                query_data,
                {'id': 'job_id'}
            )
        )

    def get_public_job_list(self, search="", time_range=TimeRange.ANY, number=100):
        """Obtains a list of public Viterbi jobs, filtering based on the search terms
        and the time range within which the job was created.

        Parameters
        ----------
        search : str, optional
            Search terms by which to filter public job list, by default ""
        time_range : .TimeRange or str, optional
            Time range by which to filter job list, by default TimeRange.ANY
        number : int, optional
            Number of job results to return in one request, by default 100

        Returns
        -------
        list
            List of ViterbiJob instances for the jobs corresponding to the search terms and in the specified time range
        """
        query = """
            query ($search: String, $timeRange: String, $first: Int){
                publicViterbiJobs (search: $search, timeRange: $timeRange, first: $first) {
                    edges {
                        node {
                            id
                            user
                            name
                            description
                            jobStatus {
                                name
                                date
                            }
                        }
                    }
                }
            }
        """

        variables = {
            "search": search,
            "time_range": time_range.value if isinstance(time_range, TimeRange) else time_range,
            "first": number
        }

        data = self.request(query=query, variables=variables)

        if not data['public_viterbi_jobs']['edges']:
            logger.info('Job search returned no results.')
            return []

        return [self._get_job_model_from_query(job['node']) for job in data['public_viterbi_jobs']['edges']]

    def get_job_by_id(self, job_id):
        """Get a Viterbi job instance corresponding to a specific job ID

        Parameters
        ----------
        job_id : str
            ID of job to obtain

        Returns
        -------
        ViterbiJob
            ViterbiJob instance corresponding to the input ID
        """
        query = """
            query ($id: ID!){
                viterbiJob (id: $id) {
                    id
                    name
                    user
                    description
                    jobStatus {
                        name
                        date
                    }
                }
            }
        """

        variables = {
            "id": job_id
        }

        data = self.request(query=query, variables=variables)

        if not data['viterbi_job']:
            logger.info('No job matching input ID was returned.')
            return None

        return self._get_job_model_from_query(data['viterbi_job'])

    def get_user_jobs(self, number=100):
        """Obtains a list of Viterbi jobs created by the user, filtering based on the search terms
        and the time range within which the job was created.

        Parameters
        ----------
        number : int, optional
            Number of job results to return in one request, by default 100

        Returns
        -------
        list
            List of ViterbiJob instances for the jobs corresponding to the search terms and in the specified time range
        """
        query = """
            query ($first: Int){
                viterbiJobs (first: $first){
                    edges {
                        node {
                            id
                            name
                            user
                            description
                            jobStatus {
                                name
                                date
                            }
                        }
                    }
                }
            }
        """

        variables = {
            "first": number
        }

        data = self.request(query=query, variables=variables)

        return [self._get_job_model_from_query(job['node']) for job in data['viterbi_jobs']['edges']]

    def _get_files_by_job_id(self, job_id):
        query = """
            query ($jobId: ID!) {
                viterbiResultFiles (jobId: $jobId) {
                    files {
                        path
                        isDir
                        fileSize
                        downloadToken
                    }
                }
            }
        """

        variables = {
            "job_id": job_id
        }

        data = self.request(query=query, variables=variables)

        file_list = FileReferenceList()
        for file_data in data['viterbi_result_files']['files']:
            if file_data['is_dir']:
                continue
            file_data.pop('is_dir')
            file_list.append(
                FileReference(
                    **file_data,
                    job_id=job_id,
                )
            )

        return file_list, False

    def get_files_by_reference(self, file_references):
        """Obtains file data when provided a FileReferenceList

        Parameters
        ----------
        file_references : FileReferenceList
            Contains the :class:`FileReference` objects for which to download the contents

        Returns
        -------
        list
            List of tuples containing the file path and file contents as a byte string
        """
        batched = file_references.batched

        file_ids = [
            self._get_download_ids_from_tokens(job_id, job_files.get_tokens())
            for job_id, job_files in batched.items()
        ]

        file_ids = list(itertools.chain.from_iterable(file_ids))
        batched_files = FileReferenceList(list(itertools.chain.from_iterable(batched.values())))

        file_paths = batched_files.get_paths()
        total_size = batched_files.get_total_bytes()

        files = _download_files(_get_file_map_fn, file_ids, file_paths, total_size)

        logger.info(f'All {len(file_ids)} files downloaded!')

        return files

    def save_files_by_reference(self, file_references, root_path, preserve_directory_structure=True):
        """Save files when provided a FileReferenceList and a root path

        Parameters
        ----------
        file_references : FileReferenceList
            Contains the :class:`FileReference` objects for which to save the associated files
        root_path : str or ~pathlib.Path
            Directory into which to save the files
        preserve_directory_structure : bool, optional
            Remove any directory structure for the downloaded files, by default True
        """
        batched = file_references.batched

        file_ids = [
            self._get_download_ids_from_tokens(job_id, job_files.get_tokens())
            for job_id, job_files in batched.items()
        ]

        file_ids = list(itertools.chain.from_iterable(file_ids))
        batched_files = FileReferenceList(list(itertools.chain.from_iterable(batched.values())))

        file_paths = batched_files.get_output_paths(root_path, preserve_directory_structure)
        total_size = batched_files.get_total_bytes()

        _download_files(_save_file_map_fn, file_ids, file_paths, total_size)

        logger.info(f'All {len(file_ids)} files saved!')

    def _get_download_id_from_token(self, job_id, file_token):
        """Get a single file download id for a file download token

        Parameters
        ----------
        job_id : str
            Job id which owns the file token

        file_token : str
            Download token for the desired file

        Returns
        -------
        str
            Download id for the desired file
        """
        return self._get_download_ids_from_tokens(job_id, [file_token])[0]

    def _get_download_ids_from_tokens(self, job_id, file_tokens):
        """Get many file download ids for a list of file download tokens

        Parameters
        ----------
        job_id : str
            Job id which owns the file token

        file_tokens : list
            Download tokens for the desired files

        Returns
        -------
        list
            List of download ids for the desired files
        """
        query = """
            mutation ResultFileMutation($input: GenerateFileDownloadIdsInput!) {
                generateFileDownloadIds(input: $input) {
                    result
                }
            }
        """

        variables = {
            "input": {
                "job_id": job_id,
                "download_tokens": file_tokens
            }
        }

        data = self.request(query=query, variables=variables)

        return data['generate_file_download_ids']['result']



import pytest
from pathlib import Path
from gwlab_viterbi_python.utils import identifiers


@pytest.fixture
def setup_paths():
    return {
        'ini': Path('test.ini'),
        'dir_ini': Path('this/is/a/test.ini'),

        'no_suffix': Path('this/is/a/test'),
        'ini_dir': Path('this/is/not/a/ini'),
        'dat_dir': Path('this/is/not/a/dat'),

        'candidates': Path('results_a0_phase_loglikes_scores.dat'),
        'dir_candidates': Path('this/is/results_a0_phase_loglikes_scores.dat'),
        'candidates_bad_ext': Path('this/is/results_a0_phase_loglikes_scores.png'),
        'candidates_bad_pattern': Path('results_a1_phase_loglikes_scores.dat'),
    }


@pytest.fixture
def setup_identifiers():
    return [
        (
            identifiers.ini_file,
            ['ini', 'dir_ini']
        ),
        (
            identifiers.candidates_file,
            ['candidates', 'dir_candidates']
        )
    ]


@pytest.fixture
def check_identifier(setup_paths):
    def _check_identifier(identifier, true_path_keys):
        true_paths = [value for key, value in setup_paths.items() if key in true_path_keys]
        false_paths = [value for key, value in setup_paths.items() if key not in true_path_keys]

        for path in true_paths:
            assert identifier(path) is True

        for path in false_paths:
            assert identifier(path) is False

    return _check_identifier


def test_identifiers(setup_identifiers, check_identifier):
    for identifier, true_path_keys in setup_identifiers:
        check_identifier(identifier, true_path_keys)



from .utils import file_filters

from gwdc_python.jobs import JobBase
from gwdc_python.logger import create_logger

logger = create_logger(__name__)


class ViterbiJob(JobBase):
    """
    ViterbiJob class is useful for interacting with the Viterbi jobs returned from a call to the GWCloud API.
    It is primarily used to store job information and obtain files related to the job.

    Parameters
    ----------
    client : ~gwlab_viterbi_python.gwlab_viterbi.GWLabViterbi
        A reference to the GWLabViterbi object instance from which the ViterbiJob was created
    job_id : str
        The id of the Viterbi job, required to obtain the files associated with it
    name : str
        Job name
    description : str
        Job description
    user : str
        User that ran the job
    job_status : dict
        Status of job, should have 'name' and 'date' keys corresponding to the status code and when it was produced
    kwargs : dict, optional
        Extra arguments, stored in `other` attribute
    """

    FILE_LIST_FILTERS = {
        'ini': file_filters.ini_filter,
        'candidates': file_filters.candidates_filter
    }

    def __init__(self, client, job_id, name, description, user, job_status, **kwargs):
        super().__init__(client, job_id, name, description, user, job_status)
        self.other = kwargs



from dataclasses import dataclass


@dataclass
class DataInput:
    """Data input class
    """
    data_choice: str = 'real'
    source_dataset: str = 'o3'


@dataclass
class DataParametersInput:
    """Data parameters input class
    """
    start_frequency_band: str = "188.0"
    min_start_time: str = "1238166483"
    max_start_time: str = "1254582483"
    asini: str = "0.01844"
    freq_band: str = "1.2136296"
    alpha: str = "4.974817413935078"
    delta: str = "-0.4349442914295658"
    orbit_tp: str = "1238161512.786"
    orbit_period: str = "4995.263"
    drift_time: str = "864000"
    d_freq: str = "5.78703704e-07"


@dataclass
class SearchParametersInput:
    """Search parameters input class
    """
    search_start_time: str = "1238166483"
    search_t_block: str = "864000"
    search_central_a0: str = "0.01844"
    search_a0_band: str = "0.00012"
    search_a0_bins: str = "1"
    search_central_p: str = "4995.263"
    search_p_band: str = "0.003"
    search_p_bins: str = "1"
    search_central_orbit_tp: str = "1238160263.9702501"
    search_orbit_tp_band: str = "260.8101737969591"
    search_orbit_tp_bins: str = "9"
    search_l_l_threshold: str = "296.27423"



import pytest
from gwlab_viterbi_python import FileReference, FileReferenceList
from gwlab_viterbi_python.utils import file_filters


@pytest.fixture
def ini():
    return FileReferenceList([
        FileReference(
            path='arbitrary/dir/test1.ini',
            file_size='1',
            download_token='test_token_1',
            job_id='id'
        ),
        FileReference(
            path='arbitrary/dir/test2.ini',
            file_size='1',
            download_token='test_token_2',
            job_id='id'
        ),
    ])


@pytest.fixture
def candidates():
    return FileReferenceList([
        FileReference(
            path='results_a0_phase_loglikes_scores.dat',
            file_size='1',
            download_token='test_token_3',
            job_id='id'
        ),
    ])


@pytest.fixture
def extra():
    return FileReferenceList([
        FileReference(
            path='extra/ini.png',
            file_size='1',
            download_token='test_token_4',
            job_id='id'
        ),
        FileReference(
            path='extra_scores.dat',
            file_size='1',
            download_token='test_token_5',
            job_id='id'
        ),
        FileReference(
            path='an/arbitrary.extra',
            file_size='1',
            download_token='test_token_6',
            job_id='id'
        ),
    ])


@pytest.fixture
def full(ini, candidates, extra):
    return ini + candidates + extra


def test_ini_file_filter(full, ini):
    sub_list = file_filters.ini_filter(full)
    assert file_filters.sort_file_list(sub_list) == file_filters.sort_file_list(ini)


def test_candidates_file_filter(full, candidates):
    sub_list = file_filters.candidates_filter(full)
    assert file_filters.sort_file_list(sub_list) == file_filters.sort_file_list(candidates)



import pytest
from dataclasses import asdict
from tempfile import TemporaryFile
from gwdc_python.files import FileReference, FileReferenceList
from gwdc_python.helpers import JobStatus, TimeRange

from gwlab_viterbi_python import GWLabViterbi, ViterbiJob
from gwlab_viterbi_python.utils.file_download import _get_file_map_fn, _save_file_map_fn
from gwlab_viterbi_python.utils.inputs import DataInput, DataParametersInput, SearchParametersInput


@pytest.fixture
def setup_gwl_request(mocker):
    def mock_init(self, token, endpoint):
        pass

    mock_request = mocker.Mock()
    mocker.patch('gwlab_viterbi_python.gwlab_viterbi.GWDC.__init__', mock_init)
    mocker.patch('gwlab_viterbi_python.gwlab_viterbi.GWDC.request', mock_request)

    return GWLabViterbi(token='my_token'), mock_request


@pytest.fixture
def setup_mock_download_fns(mocker):
    def get_mock_ids(job_id, tokens):
        return [f'{job_id}{i}' for i, _ in enumerate(tokens)]

    mock_ids = mocker.Mock(side_effect=get_mock_ids)
    return (
        mocker.patch('gwlab_viterbi_python.gwlab_viterbi.GWLabViterbi._get_download_ids_from_tokens', mock_ids),
        mocker.patch('gwlab_viterbi_python.gwlab_viterbi._download_files')
    )


@pytest.fixture
def job_data():
    return [
        {
            "id": 1,
            "name": "test_name_1",
            "description": "test description 1",
            "user": "Test User1",
            "job_status": {
                "name": "Completed",
                "date": "2021-01-01"
            }
        },
        {
            "id": 2,
            "name": "test_name_2",
            "description": "test description 2",
            "user": "Test User2",
            "job_status": {
                "name": "Completed",
                "date": "2021-02-02"
            }
        },
        {
            "id": 3,
            "name": "test_name_3",
            "description": "test description 3",
            "user": "Test User3",
            "job_status": {
                "name": "Error",
                "date": "2021-03-03"
            }
        }
    ]


def multi_job_request(query_name, return_job_data):
    return {
        query_name: {
            "edges": [{"node": job_datum} for job_datum in return_job_data],
        }
    }


@pytest.fixture
def job_file_data():
    return [
        {
            "path": "path/to/test.png",
            "file_size": "1",
            "download_token": "test_token_1",
            "is_dir": False
        },
        {
            "path": "path/to/test.json",
            "file_size": "10",
            "download_token": "test_token_2",
            "is_dir": False
        },
        {
            "path": "path/to/test",
            "file_size": "100",
            "download_token": "test_token_3",
            "is_dir": True
        }
    ]


@pytest.fixture
def test_files():
    return FileReferenceList([
        FileReference(
            path='test/path_1.png',
            file_size=1,
            download_token='test_token_1',
            job_id='id1',
        ),
        FileReference(
            path='test/path_2.png',
            file_size=1,
            download_token='test_token_2',
            job_id='id1',
        ),
        FileReference(
            path='test/path_3.png',
            file_size=1,
            download_token='test_token_3',
            job_id='id2',
        ),
        FileReference(
            path='test/path_4.png',
            file_size=1,
            download_token='test_token_4',
            job_id='id2',
        ),
        FileReference(
            path='test/path_5.png',
            file_size=1,
            download_token='test_token_5',
            job_id='id3',
        ),
        FileReference(
            path='test/path_6.png',
            file_size=1,
            download_token='test_token_6',
            job_id='id3',
        ),
    ])


def test_get_job_model_from_query(setup_gwl_request, job_data):
    gwl, _ = setup_gwl_request
    single_job_data = job_data[0]

    assert gwl._get_job_model_from_query(None) is None

    expected = ViterbiJob(
        client=gwl,
        job_id=single_job_data["id"],
        name=single_job_data["name"],
        description=single_job_data["description"],
        user=single_job_data["user"],
        job_status=single_job_data["job_status"],
    )
    assert gwl._get_job_model_from_query(single_job_data) == expected


def test_get_job_by_id(setup_gwl_request, job_data):
    gwl, mock_request = setup_gwl_request
    mock_request.return_value = {"viterbi_job": None}
    assert gwl.get_job_by_id('job_id') is None

    single_job_data = job_data[0]
    mock_request.return_value = {"viterbi_job": single_job_data}

    job = gwl.get_job_by_id('job_id')

    mock_request.assert_called_with(
        query="""
            query ($id: ID!){
                viterbiJob (id: $id) {
                    id
                    name
                    user
                    description
                    jobStatus {
                        name
                        date
                    }
                }
            }
        """,
        variables={
            "id": "job_id"
        }
    )

    assert job.job_id == single_job_data["id"]
    assert job.name == single_job_data["name"]
    assert job.description == single_job_data["description"]
    assert job.status == JobStatus(
        status=single_job_data["job_status"]["name"],
        date=single_job_data["job_status"]["date"]
    )
    assert job.user == single_job_data["user"]


def test_get_user_jobs(setup_gwl_request, job_data):
    gwl, mock_request = setup_gwl_request
    mock_request.return_value = multi_job_request('viterbi_jobs', [])
    assert gwl.get_user_jobs() == []

    mock_request.return_value = multi_job_request('viterbi_jobs', job_data)
    jobs = gwl.get_user_jobs(number=100)
    mock_request.assert_called_with(
        query="""
            query ($first: Int){
                viterbiJobs (first: $first){
                    edges {
                        node {
                            id
                            name
                            user
                            description
                            jobStatus {
                                name
                                date
                            }
                        }
                    }
                }
            }
        """,
        variables={
            "first": 100
        }
    )

    assert jobs[0].job_id == job_data[0]["id"]
    assert jobs[0].name == job_data[0]["name"]
    assert jobs[0].description == job_data[0]["description"]
    assert jobs[0].status == JobStatus(
        status=job_data[0]["job_status"]["name"],
        date=job_data[0]["job_status"]["date"]
    )
    assert jobs[0].user == job_data[0]["user"]

    assert jobs[1].job_id == job_data[1]["id"]
    assert jobs[1].name == job_data[1]["name"]
    assert jobs[1].description == job_data[1]["description"]
    assert jobs[1].status == JobStatus(
        status=job_data[1]["job_status"]["name"],
        date=job_data[1]["job_status"]["date"]
    )
    assert jobs[1].user == job_data[1]["user"]

    assert jobs[2].job_id == job_data[2]["id"]
    assert jobs[2].name == job_data[2]["name"]
    assert jobs[2].description == job_data[2]["description"]
    assert jobs[2].status == JobStatus(
        status=job_data[2]["job_status"]["name"],
        date=job_data[2]["job_status"]["date"]
    )
    assert jobs[2].user == job_data[2]["user"]


def test_get_public_job_list(setup_gwl_request, job_data):
    gwl, mock_request = setup_gwl_request
    mock_request.return_value = multi_job_request('public_viterbi_jobs', [])
    assert gwl.get_public_job_list() == []

    mock_request.return_value = multi_job_request('public_viterbi_jobs', job_data)
    jobs = gwl.get_public_job_list(search="Test", time_range=TimeRange.DAY, number=100)

    mock_request.assert_called_with(
        query="""
            query ($search: String, $timeRange: String, $first: Int){
                publicViterbiJobs (search: $search, timeRange: $timeRange, first: $first) {
                    edges {
                        node {
                            id
                            user
                            name
                            description
                            jobStatus {
                                name
                                date
                            }
                        }
                    }
                }
            }
        """,
        variables={
            "search": "Test",
            "time_range": TimeRange.DAY.value,
            "first": 100
        }
    )

    assert jobs[0].job_id == job_data[0]["id"]
    assert jobs[0].name == job_data[0]["name"]
    assert jobs[0].description == job_data[0]["description"]
    assert jobs[0].status == JobStatus(
        status=job_data[0]["job_status"]["name"],
        date=job_data[0]["job_status"]["date"]
    )
    assert jobs[0].user == job_data[0]["user"]

    assert jobs[1].job_id == job_data[1]["id"]
    assert jobs[1].name == job_data[1]["name"]
    assert jobs[1].description == job_data[1]["description"]
    assert jobs[1].status == JobStatus(
        status=job_data[1]["job_status"]["name"],
        date=job_data[1]["job_status"]["date"]
    )
    assert jobs[1].user == job_data[1]["user"]

    assert jobs[2].job_id == job_data[2]["id"]
    assert jobs[2].name == job_data[2]["name"]
    assert jobs[2].description == job_data[2]["description"]
    assert jobs[2].status == JobStatus(
        status=job_data[2]["job_status"]["name"],
        date=job_data[2]["job_status"]["date"]
    )
    assert jobs[2].user == job_data[2]["user"]


def test_gwlab_files_by_job_id(setup_gwl_request, job_file_data):
    gwl, mock_request = setup_gwl_request
    mock_request.return_value = {
        "viterbi_result_files": {
            "files": job_file_data,
        }
    }

    file_list, _ = gwl._get_files_by_job_id('arbitrary_job_id')

    mock_request.assert_called_with(
        query="""
            query ($jobId: ID!) {
                viterbiResultFiles (jobId: $jobId) {
                    files {
                        path
                        isDir
                        fileSize
                        downloadToken
                    }
                }
            }
        """,
        variables={
            "job_id": "arbitrary_job_id"
        }
    )

    for i, ref in enumerate(file_list):
        job_file_data[i].pop('is_dir', None)
        assert ref == FileReference(
            **job_file_data[i],
            job_id='arbitrary_job_id',
        )


def test_gwlab_get_files_by_reference(setup_mock_download_fns, setup_gwl_request, mocker, test_files):
    mock_get_ids, mock_download_files = setup_mock_download_fns
    gwl, _ = setup_gwl_request

    mock_download_files.return_value = [(f.path, TemporaryFile()) for f in test_files]

    files = gwl.get_files_by_reference(test_files)

    mock_calls = [
        mocker.call(job_id, job_files.get_tokens())
        for job_id, job_files in test_files.batched.items()
    ]

    mock_get_ids.assert_has_calls(mock_calls)

    assert [f[0] for f in files] == test_files.get_paths()
    mock_download_files.assert_called_once_with(
        _get_file_map_fn,
        ['id10', 'id11', 'id20', 'id21', 'id30', 'id31'],
        test_files.get_paths(),
        test_files.get_total_bytes()
    )


def test_gwlab_save_batched_files(setup_mock_download_fns, setup_gwl_request, mocker, test_files):
    mock_get_ids, mock_download_files = setup_mock_download_fns
    gwl, _ = setup_gwl_request

    gwl.save_files_by_reference(test_files, 'test_dir', preserve_directory_structure=True)

    mock_calls = [
        mocker.call(job_id, job_files.get_tokens())
        for job_id, job_files in test_files.batched.items()
    ]

    mock_get_ids.assert_has_calls(mock_calls)

    mock_download_files.assert_called_once_with(
        _save_file_map_fn,
        ['id10', 'id11', 'id20', 'id21', 'id30', 'id31'],
        test_files.get_output_paths('test_dir', preserve_directory_structure=True),
        test_files.get_total_bytes()
    )


def test_gwlab_start_job(setup_gwl_request, mocker):
    gwl, mock_request = setup_gwl_request

    mock_request.return_value = {
        "new_viterbi_job": {
            "result": {
                "job_id": "test_id"
            }
        }
    }

    gwl.get_job_by_id = mocker.Mock()

    test_name = "test_name"
    test_description = "test_description"
    test_privacy = False
    test_data_input = DataInput()
    test_data_params_input = DataParametersInput()
    test_search_params_input = SearchParametersInput()

    mock_return = gwl.start_viterbi_job(
        job_name=test_name,
        job_description=test_description,
        private=test_privacy,
        data_input=test_data_input,
        data_params=test_data_params_input,
        search_params=test_search_params_input
    )
    mock_request.assert_called_with(
        query="""
            mutation NewViterbiJob($input: ViterbiJobMutationInput!){
                newViterbiJob (input: $input) {
                    result {
                        jobId
                    }
                }
            }
        """,
        variables={
            "input": {
                "start": {
                    "name": test_name,
                    "description": test_description,
                    "private": test_privacy,
                },
                "data": asdict(test_data_input),
                "data_parameters": asdict(test_data_params_input),
                "search_parameters": asdict(test_search_params_input),
            }
        }
    )

    gwl.get_job_by_id.assert_called_once_with("test_id")

    assert mock_return == gwl.get_job_by_id.return_value



from .gwlab_viterbi import GWLabViterbi
from .viterbi_job import ViterbiJob

from gwdc_python.files import FileReference, FileReferenceList
from gwdc_python.helpers import TimeRange, JobStatus


try:
    from importlib.metadata import version
except ModuleNotFoundError:
    from importlib_metadata import version
__version__ = version('gwlab_viterbi_python')



from gwdc_python.files.identifiers import match_file_suffix, match_file_name


def ini_file(file_path):
    """Checks to see if the given file path points to an ini file

    Parameters
    ----------
    file_path : ~pathlib.Path
        File path to check

    Returns
    -------
    bool
        True if path points to a ini file, False otherwise
    """
    return match_file_suffix(file_path, 'ini')


def candidates_file(file_path):
    """Checks to see if the given file path points to the candidates file

    Parameters
    ----------
    file_path : ~pathlib.Path
        File path to check

    Returns
    -------
    bool
        True if path points to candidates file, False otherwise
    """
    return match_file_name(file_path, 'results_a0_phase_loglikes_scores.dat')



import pytest
from gwlab_viterbi_python import ViterbiJob, FileReference, FileReferenceList
from gwlab_viterbi_python.utils import file_filters


@pytest.fixture
def ini():
    return FileReferenceList([
        FileReference(
            path='arbitrary/dir/test1.ini',
            file_size='1',
            download_token='test_token_1',
            job_id='id'
        ),
        FileReference(
            path='arbitrary/dir/test2.ini',
            file_size='1',
            download_token='test_token_2',
            job_id='id'
        ),
    ])


@pytest.fixture
def candidates():
    return FileReferenceList([
        FileReference(
            path='results_a0_phase_loglikes_scores.dat',
            file_size='1',
            download_token='test_token_3',
            job_id='id'
        ),
    ])


@pytest.fixture
def extra():
    return FileReferenceList([
        FileReference(
            path='extra/ini.png',
            file_size='1',
            download_token='test_token_4',
            job_id='id'
        ),
        FileReference(
            path='extra_scores.dat',
            file_size='1',
            download_token='test_token_5',
            job_id='id'
        ),
        FileReference(
            path='an/arbitrary.extra',
            file_size='1',
            download_token='test_token_6',
            job_id='id'
        ),
    ])


@pytest.fixture
def txt():
    return FileReferenceList([
        FileReference(
            path='atoms/188-0/sfts_used.txt',
            file_size='1',
            download_token='test_token_7',
            job_id='id'
        ),
    ])


@pytest.fixture
def full(ini, candidates, extra, txt):
    return ini + candidates + extra + txt


@pytest.fixture
def mock_viterbi_job(mocker):
    def viterbi_job(methods={}):
        config_dict = {f'{key}.return_value': value for key, value in methods.items()}
        return ViterbiJob(
            client=mocker.Mock(**config_dict),
            job_id='test_id',
            name='TestName',
            description='Test description',
            user='Test User',
            job_status={
                'name': 'Completed',
                'date': '2021-12-02'
            },
        )

    return viterbi_job


@pytest.fixture
def mock_viterbi_job_files(mock_viterbi_job, full):
    return mock_viterbi_job({'_get_files_by_job_id': (full, False)})


def test_viterbi_job_full_file_list(mock_viterbi_job_files, full):
    viterbi_job = mock_viterbi_job_files
    assert viterbi_job.get_full_file_list() == full

    viterbi_job.client._get_files_by_job_id.assert_called_once()


def test_viterbi_job_equality(mocker):
    job_data = {
        "client": mocker.Mock(),
        "job_id": 1,
        "name": "test_name",
        "description": "test description",
        "user": "Test User",
        "job_status": {
            "name": "Completed",
            "date": "2021-01-01"
        }
    }
    job_data_changed_id = {**job_data, "job_id": 2}
    job_data_changed_name = {**job_data, "name": "testing_name"}
    job_data_changed_user = {**job_data, "user": "Testing User"}

    assert ViterbiJob(**job_data) == ViterbiJob(**job_data)
    assert ViterbiJob(**job_data) != ViterbiJob(**job_data_changed_id)
    assert ViterbiJob(**job_data) != ViterbiJob(**job_data_changed_name)
    assert ViterbiJob(**job_data) != ViterbiJob(**job_data_changed_user)


def test_viterbi_job_file_filters(mocker, mock_viterbi_job_files, full, ini, candidates):
    viterbi_job = mock_viterbi_job_files
    assert file_filters.sort_file_list(viterbi_job.get_ini_file_list()) == file_filters.sort_file_list(ini)
    assert file_filters.sort_file_list(viterbi_job.get_candidates_file_list()) == \
        file_filters.sort_file_list(candidates)

    assert viterbi_job.client._get_files_by_job_id.call_count == 2


def test_viterbi_job_file_filters_args(mocker, mock_viterbi_job):
    mocker.patch('gwlab_viterbi_python.viterbi_job.ViterbiJob.get_full_file_list')
    viterbi_job = mock_viterbi_job({
        'save_files_by_reference': None,
        'get_files_by_reference': None,
    })
    try:
        viterbi_job.get_ini_files()
        viterbi_job.save_ini_files(root_path='.', preserve_directory_structure=True)
        viterbi_job.get_candidates_files()
        viterbi_job.save_candidates_files(root_path='.', preserve_directory_structure=True)
    except Exception as e:
        pytest.fail(f"Test failed with exception: {e}")


def test_register_file_list_filter(mock_viterbi_job_files, txt):
    viterbi_job = mock_viterbi_job_files

    def get_txt_file(file_list):
        return [f for f in file_list if f.path.suffix == '.txt']

    assert getattr(viterbi_job, 'get_txt_file_list', None) is None
    assert getattr(viterbi_job, 'get_txt_files', None) is None
    assert getattr(viterbi_job, 'save_txt_files', None) is None

    ViterbiJob.register_file_list_filter('txt', get_txt_file)

    assert getattr(viterbi_job, 'get_txt_file_list', None) is not None
    assert getattr(viterbi_job, 'get_txt_files', None) is not None
    assert getattr(viterbi_job, 'save_txt_files', None) is not None

    assert viterbi_job.get_txt_file_list() == txt



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gwlab_viterbi_python',
 'gwlab_viterbi_python.tests',
 'gwlab_viterbi_python.utils',
 'gwlab_viterbi_python.utils.tests']

package_data = \
{'': ['*']}

install_requires = \
['graphene-file-upload>=1.3.0,<2.0.0',
 'gwdc-python>=0.4.0,<0.5.0',
 'importlib-metadata>=4.5.0,<5.0.0',
 'jwt>=1.2.0,<2.0.0',
 'requests>=2.25.1,<3.0.0',
 'tqdm>=4.61.2,<5.0.0']

extras_require = \
{'docs': ['Sphinx>=4.0.2,<5.0.0', 'sphinx-rtd-theme>=0.5.2,<0.6.0']}

setup_kwargs = {
    'name': 'gwlab-viterbi-python',
    'version': '0.2.0',
    'description': 'Wrapper of GWDC API, used for interacting with the GWLab Viterbi endpoint',
    'long_description': None,
    'author': 'Thomas Reichardt',
    'author_email': 'treichardt@swin.edu.au',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from gwdc_python.files.filters import filter_file_list
from . import identifiers


def ini_filter(file_list):
    """Takes an input file list and returns a subset of that file list containing:

    - Any ini file

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        Subset of the input FileReferenceList containing only the paths that match the above config file criteria
    """
    return filter_file_list(identifiers.ini_file, file_list)[0]


def candidates_filter(file_list):
    """Takes an input file list and returns a subset of that file list containing:

    - Any file named 'results_a0_phase_loglikes_scores.dat'

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        Subset of the input FileReferenceList containing only the paths that match the above config file criteria
    """
    return filter_file_list(identifiers.candidates_file, file_list)[0]


def sort_file_list(file_list):
    """Sorts a file list based on the 'path' key of the dicts. Primarily used for equality checks.

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        A FileReferenceList containing the same members as the input,
        sorted by the path attribute of the FileReference objects
    """
    return sorted(file_list, key=lambda f: f.path)
